@@ -77,8 +77,13 @@ echoAirGetFacilityInfo <- function(output = "df", verbose = FALSE, ...) {
 
     info <- httr::content(request)
 
+    ## return the query id
     qid <- info[["Results"]][["QueryID"]]
 
+    ## return the number of records
+    n_records <- info[["Results"]][["QueryRows"]]
+    n_records <- as.numeric(n_records)
+
     ## build the output
 
     ## get qcolumns argument specific to this query
@@ -93,17 +98,56 @@ echoAirGetFacilityInfo <- function(output = "df", verbose = FALSE, ...) {
     ## if df return output from air_rest_services.get_download
     if (output == "df") {
 
+      if (n_records <= 100000) {
 
         buildOutput <- getDownload("caa",
                                    qid,
                                    qcolumns,
                                    col_types = colTypes)
-        return(buildOutput)
+      } else {
+
+        # number of pages returned is n_records/5000
+        pages <- ceiling(n_records/5000)
+        # create the progress bar
+        pb <- progress_bar$new(total = pages)
+
+        buildOutput <- getQID("caa",
+                              qid,
+                              qcolumns,
+                              page = 1)
+        pb$tick()
+
+        for (i in 2:pages) {
+          buildOutput <- bind_rows(buildOutput,
+                                   getQID("caa",
+                                          qid,
+                                          qcolumns,
+                                          page = i))
+          Sys.sleep(0.5)
+          pb$tick()
         }
+      }
+      return(buildOutput)
+    }
 
     ## if df return output from air_rest_services.get_geojson
     if (output == "sf") {
 
+      ## if returns clusters, there are to many records to
+      ## return records via geojson and the request needs to
+      ## be more specific. I'm not sure how many records are too
+      ## many. If the length of facilities == 0, it means
+      ## the query either return no records, or the request returned
+      ## clusters and we can stop the function and return a message.
+      if(length(info[["Results"]][["Facilities"]]) == 0) {
+        if(n_records > 0) {
+          stop("Too many records to return spatial a object, please subset your request and try again.")
+        }
+        if(n_records == 0) {
+          stop("No records returned in your request")
+        }
+      }
+
       buildOutput <- getGeoJson("caa",
                                 qid,
                                 qcolumns)
